import express, { Express } from 'express';
import cors from 'cors';
import session from 'express-session';
import { getRoutes } from './router';
import { RequestId, ErrorHandler, checkPermissions } from './middleware';
import { Services } from '../../app';
import { ServerConfig } from '../../config';
import { randomUUID } from 'crypto';

class Server {
  private app: Express;
  private config: ServerConfig;

  constructor(config: ServerConfig, services: Services) {
    this.config = config;

    this.app = express();
    this.app.use(express.json());
    this.app.use(
      cors({
        maxAge: 86400,
        credentials: true,
        origin: this.config.origin,
      }),
    );
    this.app.use(RequestId());
    this.app.use(
      session({
        secret: randomUUID(),
        cookie: { maxAge: 1000 * 3600 * 24 },
        resave: false,
        saveUninitialized: false,
      }),
    );
    //this.app.use(checkPermissions());
    this.app.use(getRoutes(services));
    this.app.use(ErrorHandler());
  }

  start(func: () => void): void {
    this.app.listen(this.config.port, func);
  }
}

export default Server;
